################################################################################
# enviMass 2.0 workflow ########################################################
# maintainer: Martin.Loos@eawag.ch #############################################
################################################################################

################################################################################
# required: packages ###########################################################
# clean workspace ##############################################################
rm(list=ls(all=TRUE))
# copypaste packages from c:\...\eawag\packages to folder c:\...\Program Files\R\R-X.X.X\library
# replace existing packages if asked ###########################################
# load packages ################################################################
library(xcms)
library(mgcv)
library(nontarget)
library(isopat)
# read additional R script #####################################################
source(file="C:\\Documents and Settings\\loosmart\\Desktop\\MS\\eawag\\functions.r")
################################################################################

################################################################################
# required: mzXML profile data sets: river sample & blind ######################
# = HPLC-ESI-HRMS, Thermo orbitrap, 100.000@400m/z, positive ionization ########
# set path to data set folder
path<-c("C:\\Documents and Settings\\loosmart\\Desktop\\MS\\eawag\\data\\measurements\\")
# run xcms peak picking on the profile mzXML data, use batch wrapper ###########
# check ?findPeaks.centWave for help / ignore warnings #########################
peaks<-batchpick(
  folder=path,
  ppm=5,
  peakwidth=c(20,60),
  snthresh=10,
  prefilter=c(3,2000),
  mzCenterFun="meanApex3",
  integrate=1,
  mzdiff=0.005,
  fitgauss=FALSE
);
sampled<-peaks[[1]];
blind<-peaks[[2]];
rm(peaks);
# Thus, peaklists "sampled" and "blind" are available in the workspace #####
# plot m/z vs. RT
plot(sampled[,1],sampled[,2],xlab="m/z",ylab="Intensity",pch=19,cex=0.5,col="green");
points(blind[,1],blind[,2],pch=19,cex=0.5,col="blue");
# plot m/z vs. peak intensity
plot(sampled[,1],sampled[,3],xlab="m/z",ylab="RT [s]",pch=19,cex=0.5,col="green");
points(blind[,1],blind[,3],pch=19,cex=0.5,col="blue");
################################################################################
# required: list of isotopes from package nontarget ############################
data(iso_list,package="nontarget");
################################################################################
# required: list of adducts from package nontarget #############################
data(adduct_list,package="nontarget");
################################################################################
# required: define standard adduct #############################################
add<-"M+H"
################################################################################
# required: internal standards (IS) doted to sample ############################
IS<-read.table("C:\\Documents and Settings\\loosmart\\Desktop\\MS\\UFZ\\data\\IS.txt",header=TRUE);
# calculate monoisotopic mass of IS chemical formulas, use package isopat ######
mzIS<-c();
RTIS<-c();
for(i in 1:length(IS[,3])){
  mzIS<-c(mzIS,prof(isopattern(iso_list,as.character(IS[i,3]),0.5),adduct=add,adduct_list,
  res=4,ppm=FALSE,sdfactor=1,stickfactor=4,getprof="centroid",cutoff=0.001)[1,1]);
  RTIS<-c(RTIS,IS[i,4]*60);  # convert IS retention time from [min] to [s]
};
################################################################################

################################################################################
# data preprocessing ###########################################################
################################################################################
# recalibrate m/z of sample peaks by internal standard (IS) m/z ################
tolret<-rep(30,length(RTIS));
sampled_rec<-recall(sampled,mzIS,tolmz=10,ppm=TRUE,RTIS,tolret,what="mass",one=TRUE,knot=6,plotit=TRUE);
# allign blind m/z to sampled m/z ##############################################
mz<-sampled_rec[order(sampled_rec[,2],decreasing=TRUE)[1:2000],1];
ret<-sampled_rec[order(sampled_rec[,2],decreasing=TRUE)[1:2000],3];
tolret<-rep(60,length(ret));
blind_allign<-recall(blind,mz,tolmz=10,ppm=TRUE,ret,tolret,what="mass",one=TRUE,knot=6,plotit=TRUE);
# subtract blind from sample ###################################################
length(sampled[,1]); # before
sampled_sub<-sub_peak(sampled_rec,blind_allign,tolmz=5,ppm=TRUE,tolret=30,cutoff=FALSE)
length(sampled_sub[,1]); # after
# plot m/z vs. RT
plot(sampled_rec[,1],sampled_rec[,3],xlab="m/z",ylab="RT [s]",pch=19,cex=0.5,col="green");
points(blind_allign[,1],blind_allign[,3],pch=19,cex=0.5,col="blue");
points(sampled_sub[,1],sampled_sub[,3],pch=19,cex=0.5,col="red");
# plot m/z vs. peak intensity
plot(sampled_rec[,1],sampled_rec[,2],xlab="m/z",ylab="Intensity",pch=19,cex=0.5,col="green");
points(blind_allign[,1],blind_allign[,2],pch=19,cex=0.5,col="blue");
points(sampled_sub[,1],sampled_sub[,2],pch=19,cex=0.5,col="red");
# plot m/z vs. log10( peak intensity )
plot(sampled_rec[,1],log10(sampled_rec[,2]),xlab="m/z",ylab="log10(Intensity)",pch=19,cex=0.5,col="green");
points(blind_allign[,1],log10(blind_allign[,2]),pch=19,cex=0.5,col="blue");
points(sampled_sub[,1],log10(sampled_sub[,2]),pch=19,cex=0.5,col="red");
################################################################################

################################################################################
# componentization, use package nontarget ######################################
################################################################################
# (1) remove satelite peaks ####################################################
# sampled_sub<-rm.sat(sampled_sub,dmz=0.3,drt=20,intrat=0.05,spar=0.8,corcut=-1000,plotit=TRUE);
# sampled_sub<-sampled_sub[sampled_sub[,4],1:3];
################################################################################
# (2) histogram of m/z differences among peaks: indicates adducts and isotopes #
plotdiff(sampled_sub, histbreaks = 40000, rttol = c(0, 0), mztol = c(0.5, 100), plotit = TRUE)
################################################################################
# (3) run isotope pattern grouping #############################################
# (3.1) define isotopes and charge argument ####################################
# for help, see ?make.iso
iso<-make.isos(iso_list,charges=c(2),elements=c("C","Cl","Br","N","S","K"),
rm.isotopes=c("33S","36S","40K"),rm.charges=c(2,2,2));
# (3.2) run isotope grouping ###################################################
# for help, see ?pattern.search
pattern<-pattern.search(
  sampled_sub,
  iso,
  cutint=50000,
  rttol=c(-10,10),
  mztol=2,
  mzfrac=0.1,
  ppm=TRUE,
  inttol=0.2,
  rules=c(TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE),
  #rules=c(FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE),
  deter=FALSE,
  entry=50
);
# (3.3) plot results ###########################################################
plotisotopes(pattern);
plotdefect(pattern,elements=c("C"));
plotdefect(pattern,elements=c("N"));
plotdefect(pattern,elements=c("S"));
plotdefect(pattern,elements=c("Cl"));
################################################################################
# (4.1) run grouping of peaks for different adducts ############################
# of the same candidate molecule ###############################################
# for help, see ?adduct.search
adduct<-adduct.search(
  sampled_sub,
  adduct_list,
  rttol=10,
  mztol=3,
  massfrac=0.1,
  ppm=TRUE,
  adducts=c("M+K","M+H","M+Na","M+NH4"),
  ion_mode="positive",
  entry=50
);
# (4.2) plot results ###########################################################
plotadduct(adduct);
################################################################################
# (5) show single pattern group and its relation to adduct groups ##############
plotall(pattern,adduct);
plotgroup(pattern,adduct,groupID=1,massrange=10,allmass=FALSE);
################################################################################
# (6.1) Screen for homologue series ############################################
# for help, see ?homol.search
homol<-homol.search(
  sampled_sub,
  iso_list,
  minmz=6,
  maxmz=60,
  minrt=10,
  maxrt=1.8*60,
  ppm=TRUE,
  mztol=3.5,
  rttol=0.4,
  minlen=4,
  plotit=TRUE,
  restr=FALSE
);
# (6.2) Plot results ###########################################################
plothomol(homol,xlim=FALSE,ylim=FALSE,plotlegend=TRUE);
################################################################################
# (7.1) Combine grouping results to components #################################
comp<-combine(
	pattern,
	adduct,
	homol,
	dont=FALSE,
	rules=c(TRUE,FALSE,FALSE)
);
# (7.2) plot results ###########################################################
plotisotopes(comp);
plotcomp(comp,compoID=2,peakID=FALSE);
################################################################################

